@@ -3,7 +3,7 @@
 
    This file is part of drbd by Philipp Reisner.
 
-   Copyright (C) 2003-2006, Philipp Reisner <philipp.reisner@linbit.com>.
+   Copyright (C) 2003-2004, Philipp Reisner <philipp.reisner@linbit.com>.
         Initial author.
 
    drbd is free software; you can redistribute it and/or modify
@@ -64,7 +64,7 @@ FILE *m_popen(int *pid,char** argv)
     close(pipes[0]); // close reading end
     dup2(pipes[1],1); // 1 = stdout
     close(pipes[1]);
-    execvp(argv[0],argv);
+    execv(argv[0],argv);
     fprintf(stderr,"Can not exec");
     exit(E_exec_error);
   }
@@ -337,7 +337,6 @@ int adm_adjust(struct d_resource* res,char* unused)
     do_connect |= check_opt_d(in,"ping-int",1,"sec",res->net_options);
     do_connect |= check_opt_d(in,"max-epoch-size",1,"",res->net_options);
     do_connect |= check_opt_d(in,"max-buffers",1,"",res->net_options);
-    do_connect |= check_opt_d(in,"unplug-watermark",1,"",res->net_options);
     do_connect |= check_opt_d(in,"sndbuf-size",1,"",res->net_options);
     do_connect |= check_opt_d(in,"ko-count",1,"",res->net_options);
     do_connect |= check_opt_s(in,"on-disconnect",res->net_options);
